/* Mix Statistics (mix_stats.c) */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "cleanup.h"

typedef struct {
  unsigned int uv_errs, v_errs, doubles, halves, both_v, both_v_pge, both_uv;
  float uverr_duration, verr_duration;
  double sum_d, sum_dpge, sum_dpge_sqr;
} STATISTICS_;

void initialise_statistics (p_stats)
     STATISTICS_ *p_stats;
{

  p_stats->uv_errs = 0;
  p_stats->v_errs = 0;
  p_stats->doubles = 0;
  p_stats->halves = 0;
  p_stats->both_v = 0;
  p_stats->both_v_pge = 0;
  p_stats->both_uv = 1;
  p_stats->uverr_duration = 0.0;
  p_stats->verr_duration = 0.0;
  p_stats->sum_d = 0.0;
  p_stats->sum_dpge = 0.0;
  p_stats->sum_dpge_sqr = 0.0;

}

void main (argc, argv)
     int argc;
     char *argv[];
{

  FILE *stats_file;
  STATISTICS_ total_stats, in_stats;
  double uv_dur = 0.0, v_dur = 0.0;

  initialise_statistics (&total_stats);
  if (argc != 4) {
    fprintf (stderr, "mix_stats: usage: mix_stats stats_file uv_dur v_dur\n");
    exit (-1);
  }
  if ((stats_file = fopen (argv[1], "rb")) == NULL) {
    fprintf (stderr, "mix_stats: cannot open file %s\n", argv[1]);
    exit (-1);
  }
  uv_dur = atof (argv[2]);
  v_dur = atof (argv[3]);
  while (fread ((STATISTICS_ *) &in_stats, sizeof (STATISTICS_), 1,
		stats_file)) {
    total_stats.uv_errs += in_stats.uv_errs;
    total_stats.v_errs += in_stats.v_errs;
    total_stats.doubles += in_stats.doubles;
    total_stats.halves += in_stats.halves;
    total_stats.both_v += in_stats.both_v;
    total_stats.both_v_pge += in_stats.both_v_pge;
    total_stats.both_uv += in_stats.both_uv;
    total_stats.uverr_duration += in_stats.uverr_duration;
    total_stats.verr_duration += in_stats.verr_duration;
    total_stats.sum_d += in_stats.sum_d;
    total_stats.sum_dpge += in_stats.sum_dpge;
    total_stats.sum_dpge_sqr += in_stats.sum_dpge_sqr;
  }
  printf ("Mix_stats: %s\n", argv[1]);
  printf (" Unvoiced error:\t%.2f %%\n", total_stats.uverr_duration * 100.0 /
	  uv_dur);
  printf (" Voiced error:\t%.2f %%\n", total_stats.verr_duration * 100.0 /
	  v_dur);
  printf (" Doubles:\t%.2f %%\n", total_stats.doubles * 100.0 /
	  total_stats.both_v);
  printf (" Halves:\t%.2f %%\n", total_stats.halves * 100.0 /
	  total_stats.both_v);
  printf (" Contour-deviation bar gross errors:\n    Average:\t\t%.2f Hz\n",
	  total_stats.sum_dpge / total_stats.both_v_pge);
  printf ("    Standard deviation:\t%.2f Hz\n",
	  sqrt ((total_stats.sum_dpge_sqr - total_stats.sum_dpge *
		 total_stats.sum_dpge / total_stats.both_v_pge) /
		(total_stats.both_v_pge - 1)));
  exit (0);

}
